@@ -16,6 +16,7 @@ from rteval.modules import rtevalModulePrototype
 from rteval.systopology import cpuinfo, SysTopology
 from rteval.cpulist_utils import expand_cpulist, collapse_cpulist
 
+
 class TLRunData:
     ''' class to store instance data from a timerlat run '''
     def __init__(self, coreid, datatype, priority, logfnc):
@@ -188,6 +189,15 @@ class Timerlat(rtevalModulePrototype):
         self.__cpus = [str(c) for c in expand_cpulist(self.__cpulist)]
         self.__numcores = len(self.__cpus)
 
+        # Has tracing been triggered
+        self.__stoptrace = False
+        # This stores the output from rtla
+        self.__posttrace = ""
+        # Stop Trace Data
+        self.__stdata = {}
+        # Stop Trace Cpu
+        self.stcpu = -1
+
         self.__timerlat_out = None
         self.__timerlat_err = None
         self.__started = False
@@ -206,6 +216,7 @@ class Timerlat(rtevalModulePrototype):
                                                   logfnc=self._log)
         self.__timerlatdata['system'].description = (f"({self.__numcores} cores) ") + info['0']['model name']
         self._log(Log.DEBUG, f"system using {self.__numcores} cpu cores")
+        self.set_latency_test()
 
 
     def _WorkloadSetup(self):
@@ -215,13 +226,25 @@ class Timerlat(rtevalModulePrototype):
         self._setReady()
 
     def _WorkloadPrepare(self):
-        self.__cmd = ['rtla', 'timerlat', 'hist', '-P', f'f:{int(self.__priority)}', '-u']
+        self.__interval = 'interval' in self.__cfg and f'-p{int(self.__cfg.interval)}' or ""
+        self.__cmd = ['rtla', 'timerlat', 'hist', self.__interval, '-P', f'f:{int(self.__priority)}', '-u']
         self.__cmd.append(f'-c{self.__cpulist}')
         self.__cmd.append(f'-E{self.__buckets}')
+
+        if self.__cfg.stoptrace:
+            self.__cmd.append(f"-T{int(self.__cfg.stoptrace)}")
+
+        if self.__cfg.trace:
+            if not self.__cfg.stoptrace:
+                self._log(Log.WARN, f'Ignoring trace={self.__cfg.trace}, because stoptrace not invoked')
+            else:
+                self.__cmd.append(f'-t={self.__cfg.trace}')
+
         self._log(Log.DEBUG, f'self.__cmd = {self.__cmd}')
         self.__timerlat_out = tempfile.SpooledTemporaryFile(mode='w+b')
         self.__timerlat_err = tempfile.SpooledTemporaryFile(mode='w+b')
 
+
     def _WorkloadTask(self):
         if self.__started:
             return
@@ -230,14 +253,11 @@ class Timerlat(rtevalModulePrototype):
 
         self.__timerlat_out.seek(0)
         self.__timerlat_err.seek(0)
-        try:
-            self.__timerlat_process = subprocess.Popen(self.__cmd,
-                                                       stdout=self.__timerlat_out,
-                                                       stderr=self.__timerlat_err,
-                                                       stdin=None)
-            self.__started = True
-        except OSError:
-            self.__started = False
+        self.__timerlat_process = subprocess.Popen(self.__cmd,
+                                                   stdout=self.__timerlat_out,
+                                                   stderr=self.__timerlat_err,
+                                                   stdin=None)
+        self.__started = True
 
     def WorkloadAlive(self):
         if self.__started:
@@ -252,8 +272,14 @@ class Timerlat(rtevalModulePrototype):
             os.kill(self.__timerlat_process.pid, signal.SIGINT)
             time.sleep(2)
 
+
         # Parse histogram output
         self.__timerlat_out.seek(0)
+
+        blocking_thread_detected = False
+        softirq_interference_detected = False
+        irq_interference_detected = False
+
         for line in self.__timerlat_out:
             line = bytes.decode(line)
 
@@ -261,6 +287,83 @@ class Timerlat(rtevalModulePrototype):
             if not line:
                 continue
 
+            # Parsing if stoptrace has been invoked
+            if self.__stoptrace:
+                self.__posttrace += line
+                line = line.strip()
+                fields = line.split()
+                if not line:
+                    continue
+                if line.startswith("##") and fields[1] == "CPU":
+                    self.stcpu = int(fields[2])
+                    self._log(Log.DEBUG, f"self.stcpu = {self.stcpu}")
+                    self.__stdata[self.stcpu] = {}
+                    continue
+                if self.stcpu == -1:
+                    self._log(Log.WARN, "Stop trace has been invoked, but a stop cpu has not been identified.")
+                    continue
+                if line.startswith('------------------'):
+                    blocking_thread_detected = False
+                    softirq_interference_detected = False
+                    irq_interference_detected = False
+                    continue
+
+                # work around rtla not printing ':' after all names
+                if line.startswith('Softirq interference'):
+                    name = 'Softirq_interference'
+                elif line.startswith('IRQ interference'):
+                    name = 'IRQ_interference'
+                else:
+                    name = ''.join(line.split(':')[0]).replace(' ', '_')
+                self._log(Log.DEBUG, f"name={name}")
+
+                if name in ['Thread_latency']:
+                    latency = fields[-3]
+                    percent = fields[-1].strip('()%')
+                    self._log(Log.DEBUG, f'{name} = ({latency}, {percent})')
+                    self.__stdata[self.stcpu][name] = (latency, percent)
+                    continue
+                if name in ['Timerlat_IRQ_duration', 'IRQ_handler_delay', 'Blocking_thread', 'IRQ_interference', 'Softirq_interference']:
+                    latency = fields[-4]
+                    percent = fields[-2].strip('(')
+                    if name == 'IRQ_handler_delay' and fields[3] == '(exit':
+                        name = 'IRQ_handler_delay_exit_from_idle'
+                    self._log(Log.DEBUG, f'{name} = ({latency}, {percent})')
+                    self.__stdata[self.stcpu][name] = (latency, percent)
+                    detected = {'Blocking_thread' : (True, False, False),
+                                'IRQ_interference' : (False, True, False),
+                                'Softirq_interference' : (False, False, True) }
+                    if name in ('Blocking_thread', 'IRQ_interference', 'Softirq_interference'):
+                        blocking_thread_detected, irq_interference_detected, softirq_interference_detected = detected.get(name)
+                    continue
+                if name in ["IRQ_latency", "Previous_IRQ_interference"]:
+                    latency = fields[-2]
+                    self._log(Log.DEBUG, f'{name} = {fields[-2]}')
+                    self.__stdata[self.stcpu][name] = fields[-2]
+                    continue
+                if blocking_thread_detected or softirq_interference_detected or irq_interference_detected:
+                    if blocking_thread_detected:
+                        field_name = "blocking_thread"
+                    elif softirq_interference_detected:
+                        field_name = "softirq_interference"
+                    elif irq_interference_detected:
+                        field_name = "irq_interference"
+                    thread = " ".join(fields[0:-2])
+                    latency = fields[-2]
+                    self._log(Log.DEBUG, f"{field_name} += [({thread}, {latency})]")
+                    self.__stdata[self.stcpu].setdefault(field_name, [])
+                    self.__stdata[self.stcpu][field_name] += [(thread, latency)]
+                    continue
+                if name == "Max_timerlat_IRQ_latency_from_idle":
+                    latency = fields[-5]
+                    max_timerlat_cpu = int(fields[-1])
+                    self._log(Log.DEBUG, f'self.__stdata[{max_timerlat_cpu}][{name}] = {latency}')
+                    self.__stdata.setdefault(max_timerlat_cpu, {})
+                    self.__stdata[max_timerlat_cpu][name] = latency
+                else:
+                    self._log(Log.DEBUG, f'line = {line}')
+                continue
+
             if line.startswith('#'):
                 if line.startswith('# Duration:'):
                     duration = line.split()[2]
@@ -285,9 +388,13 @@ class Timerlat(rtevalModulePrototype):
             elif line.startswith('max:'):
                 #print(line)
                 continue
+            elif line.startswith('rtla timerlat hit stop tracing'):
+                self.__stoptrace = True
+                self.__posttrace += line
+                continue
             else:
-                pass
                 #print(line)
+                pass
 
             vals = line.split()
             if not vals:
@@ -301,12 +408,18 @@ class Timerlat(rtevalModulePrototype):
                 continue
 
             for i, core in enumerate(self.__cpus):
-                self.__timerlatdata[core].bucket(index, int(vals[i*3+1]),
+                # There might not be a count on every cpu if tracing invoked
+                if i*3 + 1 >= len(vals):
+                    self.__timerlatdata[core].bucket(index, 0, 0, 0)
+                    self.__timerlatdata['system'].bucket(index, 0, 0, 0)
+                else:
+                    self.__timerlatdata[core].bucket(index, int(vals[i*3+1]),
                                                  int(vals[i*3+2]),
                                                  int(vals[i*3+3]))
-                self.__timerlatdata['system'].bucket(index, int(vals[i*3+1]),
+                    self.__timerlatdata['system'].bucket(index, int(vals[i*3+1]),
                                                  int(vals[i*3+2]),
                                                  int(vals[i*3+3]))
+
         # Generate statistics for each RunData object
         for n in list(self.__timerlatdata.keys()):
             self.__timerlatdata[n].reduce()
@@ -320,6 +433,62 @@ class Timerlat(rtevalModulePrototype):
         rep_n = libxml2.newNode('timerlat')
         rep_n.newProp('command_line', ' '.join(self.__cmd))
 
+        stoptrace_invoked_n = libxml2.newNode('stoptrace_invoked')
+        if self.stcpu != -1:
+            stoptrace_invoked_n.newProp("invoked", "true")
+        else:
+            stoptrace_invoked_n.newProp("invoked", "")
+        rep_n.addChild(stoptrace_invoked_n)
+
+        if self.stcpu != -1:
+            self._log(Log.DEBUG, f'self.__stdata = {self.__stdata}')
+            for cpu in self.__stdata:
+                # This is  Max timerlat IRQ latency from idle
+                # With no other data from that cpu, so don't create a
+                # stoptrace_report for this
+                if len(self.__stdata[cpu]) == 1:
+                    continue
+                stoptrace_n = libxml2.newNode('stoptrace_report')
+                stoptrace_n.newProp("CPU", str(cpu))
+                for k, v in self.__stdata[cpu].items():
+                    self._log(Log.DEBUG, f"cpu={cpu}, k={k}, v={v}")
+                    if isinstance(v, tuple):
+                        latency = str(v[0])
+                        percent = str(v[1])
+                        cpu_n = stoptrace_n.newTextChild(None, str(k), None)
+                        n = cpu_n.newTextChild(None, "latency", latency)
+                        n.newProp('unit', 'us')
+
+                        n = cpu_n.newTextChild(None, "latency_percent", percent)
+                        n.newProp('unit', '%')
+                    elif isinstance(v, list):
+                        if k in ("blocking_thread", "softirq_interference", "irq_interference"):
+                            for name, latency in v:
+                                cpu_n = stoptrace_n.newTextChild(None, k, None)
+                                n = cpu_n.newTextChild(None, "name", name)
+                                n = cpu_n.newTextChild(None, "latency", latency)
+                                n.newProp('unit', 'us')
+                    else:
+                        if k == "Max_timerlat_IRQ_latency_from_idle":
+                            continue
+                        cpu_n = stoptrace_n.newTextChild(None, str(k), str(v))
+                        cpu_n.newProp('unit', 'us')
+                rep_n.addChild(stoptrace_n)
+
+            self._log(Log.DEBUG, f'timerlat: posttrace = \n{self.__posttrace}')
+            self._log(Log.DEBUG, 'timerlat: posttrace END')
+            for cpu in self.__stdata:
+                for k, v in self.__stdata[cpu].items():
+                    if isinstance(v, tuple):
+                        continue
+                    if k == "Max_timerlat_IRQ_latency_from_idle":
+                        max_timerlat_n = libxml2.newNode('max_timerlat_report')
+                        max_timerlat_n.newProp("CPU", str(cpu))
+                        cpu_n = max_timerlat_n.newTextChild(None, k, str(v))
+                        cpu_n.newProp('unit', 'us')
+                        rep_n.addChild(max_timerlat_n)
+            return rep_n
+
         rep_n.addChild(self.__timerlatdata['system'].MakeReport())
         for thr in self.__cpus:
             if str(thr) not in self.__timerlatdata:
@@ -329,19 +498,23 @@ class Timerlat(rtevalModulePrototype):
         return rep_n
 
 
-def ModuleInfo():
-    """ Required measurement module information """
-    return {"parallel": True,
-            "loads": True}
-
 def ModuleParameters():
     """ default parameters """
-    return {"priority": {"descr": "Run rtla timerlat with this priority",
+    return {"interval": {"descr": "Base interval or period of threads in microseconds",
+                         "default": 100,
+                         "metavar": "INTV_US"},
+            "priority": {"descr": "Run rtla timerlat with this priority",
                          "default": 95,
                          "metavar": "PRIO" },
             "buckets":  {"descr": "Number of buckets",
                          "default": 3500,
-                         "metavar": "NUM"}
+                         "metavar": "NUM" },
+            "stoptrace": {"descr": "Stop trace if thread latency higher than USEC",
+                          "default": None,
+                          "metavar": "USEC" },
+            "trace":    {"descr": "File to save trace to",
+                         "default": None,
+                         "metavar": "FILE" },
            }
 
 def create(params, logger):
@@ -367,6 +540,7 @@ if __name__ == '__main__':
 
     cfg_tl = cfg.GetSection('timerlat')
     cfg_tl.cpulist = collapse_cpulist(SysTopology().online_cpus())
+    cfg_tl.stoptrace=50
 
     RUNTIME = 10
 
